
#include <stddef.h>
#include <stdint.h>

#define COL1                    (2)                        // OUTPUT
#define COL2                    (26)                       // OUTPUT
#define COL3                    (31)                       // OUTPUT
#define ROW1                    (4)                        // INPUT
#define ROW2                    (27)                       // INPUT
#define ROW3                    (10)                       // INPUT
#define ROW4                    (6)                        // INPUT - note that this in on gpio1
#define LED_GREEN               (6)                        // OUTPUT
#define LED_ILLUM               (15)                       // OUTPUT
#define LED_RED                 (8)                        // OUTPUT
#define PIN_M1                  (16)                       // OUTPUT
#define PIN_M2                  (13)                       // OUTPUT
#define PIN_M3                  (1)                        // OUTPUT - note that this in on gpio1
#define PIN_M4                  (21)                       // OUTPUT
#define VDD2_ENABLE             (7)                        // OUTPUT - note that this in on gpio1
#define VDD3_ENABLE             (7)                        // OUTPUT
#define VDD5_ENABLE             (25)                       // OUTPUT
#define PIN_ADCENABLE           (3)                        // OUTPUT - note that this in on gpio1
#define HASPSW                 (20)                        // INPUT - HASP
#define DOORSW                  (17)                       // INPUT - KEY COMPARTMENT
#define CNTRSW                  (14)                       // INPUT - MTR CENTERED
#define Q_PON                   (22)                       // OUTPUT
#define Q_PWRKEY                (11)                       // OUTPUT - note that this is on gpio1
#define Q_STATUS                (30)                       // INPUT
#define RFIRQ                   (5)                        // INPUT
#define RTCALARM                (8)                        // INPUT - note that this is on gpio1


#define EEPROMADDR              0x0051
#define RFIDADDR                0x0050
#define RTCADDR                 0x0068
#define SIG_ADDRESS             65408                                   // the eeprom address of the SIGNATURE
#define SIG_SIZE                 (7)                                    // the size of the EEPROM signature
#define SIG_CONTENT             "QxplM75"                               // the value of the signature for versione with a 10 character boxid
#define SIG_CONTENT_OLD         "QxplM74"                               // the value of the signature for versions with an 8 character boxid
#define CMDSIZE                  200                                    // the size of the commmand buffr - needed because the buffer is shared across multiple modules
struct uart_data_t {
	                 void *fifo_reserved;
					         uint8_t data[1100];
					         uint16_t len;
                   };

typedef struct {
  uint8_t P[16 + 2];
  uint8_t S[4][256];
} BLOWFISH_CTX;
struct cmd_data_t {
	void *fifo_reserved;
	uint8_t data[100];
	uint16_t len;
};
struct ble_data_t {
	void *fifo_reserved;
	uint8_t data[200];
	uint16_t len;
};

struct bt_data_t {
	void *fifo_reserved;
	uint8_t data[200];
	uint16_t len;
};
struct BATTERY {uint32_t battdt;
                uint16_t battbef;
		            uint16_t battaft;
               };
struct RTCINFO {uint32_t timset;
	              uint32_t drift;
               };
struct decodedCmd{uint32_t date;
              uint8_t cmd;
              };

struct ACTREC {
               uint32_t date; 
	             uint16_t temp;
	             uint16_t v1;  
	             uint16_t v2;
               uint8_t err;
               uint8_t command;
	             uint8_t cmd[7];	             
               uint8_t srce;
	             uint8_t phase;
	             uint8_t  unused[11];
              };

struct PARAMS {
	       uint32_t commdt;
	       uint8_t lockid[12];
	       uint8_t crkey[10];
         struct BATTERY batt[6];
	       struct RTCINFO rtc[21];
	       uint16_t ADVINT;
	       uint8_t ERRLMT;
	       uint8_t ERRTMO;
	       uint8_t INATMO;
         uint16_t RATIO;
         uint16_t VOLUME;
         uint16_t LASTBATT;
         uint8_t TIMEZONE;
         uint8_t LOCKSTATE;
         uint16_t LOWVOLTAGE;
         uint8_t USESOUND;
         uint8_t USERFID;
         uint8_t USEAWS;
         uint8_t FWVERSION[10];
         uint8_t RFTAG[20];
         uint32_t LASTBOOT;
         uint8_t IMEI[20];
         uint8_t MOTORMODE;
         uint32_t TIMEOUTDELAY;
        };

struct decodedCmd bits2(uint32_t);
void Blowfish_Init(BLOWFISH_CTX *ctx, uint8_t *key, uint8_t keyLen);
void Blowfish_Encrypt(BLOWFISH_CTX *ctx, uint8_t *xl, uint8_t *xr);
void Blowfish_Decrypt(BLOWFISH_CTX *ctx, uint8_t *xl, uint8_t *xr);

struct RTCDATE {uint16_t second;
                uint16_t minute;
                uint16_t hour;
                uint16_t day;
                uint16_t month;
                uint16_t year;
                uint16_t weekday;
               };
struct EEPROMMAP {uint16_t addr[15];                                    // 0 is EEPROM signature address
                                                                        // 1 is the CACERT address
                                                                        // 2 is the CACERT length
                                                                        // 3 is the CLICERT address
                                                                        // 4 is the CLICERT length
                                                                        // 5 is the CLIKEY address
                                                                        // 6 is the CLIKEY length
                                                                        // 7 is the AWS IOT endpoint address
                                                                        // 8 is the AWS IOT endpoint length
                                                                        // 9 is the PARAMS address
                                                                        //10 is the LOGFILE starting address
                                                                        //11 is the ACTIVITY FILE starting address
                                                                        //12 is the MESSAGE POOL starting address
                                                                        // 2 spares
                                                                          
                 };    
struct EEPROMMSGS {
                   uint32_t fstmsg;                                             
                   uint32_t lstmsg;
                   uint32_t msgcnt;
                   uint32_t availcnt;
                   uint8_t filler[16];                                 // this insures thta messages lie on a MOD 32 boundry
                  };  
struct LOGMSG{uint32_t date;
              uint8_t data[28];
             };      

#define RS     ','
// subroutine casts
void init_adc();
uint32_t read_adc();
void get_battery();
void init_i2c();
uint32_t check_rtc_reset();
void set_rtc(struct RTCDATE date);
struct RTCDATE get_rtc();
void init_rtc();
void init_gpio();
void init_i2s();
void play_i2s_sample(uint32_t , uint8_t*);
void play_i2s_tone(int, int, int);
void motor_spin(int,int);
void bt_send_data(uint8_t *);
void proc_cmd(void);
void init_temp(void);
uint32_t read_temp(void);
void eeprom_store(uint32_t, uint32_t, void *);
void eeprom_retrieve(uint32_t, uint32_t, uint8_t *);
struct RTCDATE dt_to_disp(uint32_t);
uint32_t errmsg_del(uint32_t);
uint32_t msg_cnt(uint32_t) ; 
uint32_t init_uart(void);
void uart_write(uint8_t *);
uint8_t * uart_read_thread(void);
uint32_t xtest_spi(void);
struct LOGMSG * retrieve_msg(uint32_t, uint32_t);
uint32_t quectel_powerup(void);
uint32_t quectel_powerdown(void);
uint32_t runrfid(uint32_t);
void error_flash(uint32_t,uint32_t, uint32_t);
void check_eeprom();
struct RTCDATE qget_date(void);
uint32_t  dt_to_posix(struct RTCDATE);
void keypadsetup(void);
void setup_keyscan(void);  
void vdd2_up(void);
void vdd2_down(void);
void logmsg_add(uint32_t , uint8_t *);
void motor_cw(void);
void motor_ccw();
void play_sample1(void);
void play_sample2(void);
void play_sample3(void);
void play_sample4(void);
void play_sample5(void);
void play_sample6(void);
void play_sample7(void);
void play_sample8(void);
