/*
 * Copyright (c) 2017 comsuisse AG
 *
 * SPDX-License-Identifier: Apache-2.0
 */


#include <zephyr/kernel.h>
#include  <stdio.h>
#include <string.h>
#include "common.h"
#include <zephyr/drivers/i2c.h>
extern const struct device *i2c1;  
extern struct EEPROMMAP MAP;
extern struct PARAMS PARMS;
extern uint8_t FWVERSION[]; 
extern bool DUMMYREAD;
extern bool VDD2_UP;
union
{
	uint16_t memAddr;
	uint8_t memByte[2];
} memAddr;

void eeprom_store(uint32_t address, uint32_t size, void * data)
{volatile uint32_t eeprom_curr_page,eeprom_next_page,eeprom_end_page,bytes_left,bytes_to_write;
 int32_t ret;
 uint16_t eeprom_address;
 uint8_t buffer[130], xmit_buffer[100];                                 // 128 byte EEPROM page + address
 /*
 bool vdd2_upx;                                                         // temp for testing
 vdd2_upx=VDD2_UP;                                                      // save the state of VDD2_UP
 if(!vdd2_upx)                                                          // if VDD2 is not up
 {
     vdd2_up();                                                         // bring it up

     vdd2_down();                                                       // and power down
 }
*/
 eeprom_address=address;                                                // set up to start writing
 bytes_left=size;
 while(bytes_left>0)                                                    // write looo
    {memset(buffer,0,sizeof(buffer));                                   // clear the write buffer
     memAddr.memAddr=eeprom_address;                                    // swap address bytes
     buffer[0]=memAddr.memByte[1];
     buffer[1]=memAddr.memByte[0];
     eeprom_curr_page=eeprom_address/128;                               // calculate the current eeprom page
     eeprom_next_page=(eeprom_address+bytes_left-1)/128;                    // see if  this is the last write
     if(eeprom_next_page==eeprom_curr_page)                             // if it is
        {memmove(&buffer[2],data,bytes_left);                           // the balance of the data can
         bytes_to_write=bytes_left+2;                                   //    written out, setup the write
         bytes_left=0;
        }                                                               
     else {eeprom_end_page=eeprom_curr_page*128+127;                    // otherwise calcuate how many
           bytes_to_write=eeprom_end_page-eeprom_address+3;             // bytes to write
           memmove(&buffer[2],data,(bytes_to_write-2));                 // build the buffer
           bytes_left=bytes_left-bytes_to_write+2;                      // decrement bytes left
           data=data+(bytes_to_write-2);                                // move the pointer
          }
      k_sleep(K_MSEC(4));                                               // sleep a few milliseconds
      ret=i2c_write(i2c1,buffer,bytes_to_write,EEPROMADDR);             // and write the buffer out to the EEPROM
      if(ret!=0)
           {
                 sprintf(xmit_buffer,"EEPROM WRITE FAIL AT %d",address);
                 bt_send_data(xmit_buffer);
                 if(!DUMMYREAD)
                      error_flash(1,0,1);
           }
      eeprom_address=eeprom_address+bytes_to_write-2;
    }
}

void eeprom_retrieve(uint32_t address, uint32_t size, uint8_t *data)
{volatile uint32_t eeprom_curr_page,eeprom_next_page,eeprom_end_page,bytes_left,bytes_to_read;
 volatile int32_t ret;
 uint16_t ADDR=EEPROMADDR;
 uint16_t eeprom_address;
 uint8_t write_buffer[2], read_buffer[128],buffer[100];;
 eeprom_address=address;                                                // set up to start writing
 /*
 bool vdd2_upx;                                                         // temp for testing
 vdd2_upx=VDD2_UP;                                                      // save the state of VDD2_UP
 if(!vdd2_upx)                                                          // if VDD2 is not up
 {
     vdd2_up();                                                         // bring it up

     vdd2_down();                                                       // and power down
 }
*/
 bytes_left=size;
 while(bytes_left>0)                                                    // read loop
    {memset(read_buffer,0,sizeof(read_buffer));                         // clear the write buffer
     memAddr.memAddr=eeprom_address;                                    // swap address bytes
     write_buffer[0]=memAddr.memByte[1];
     write_buffer[1]=memAddr.memByte[0];
     eeprom_curr_page=eeprom_address/128;                               // calculate the current eeprom page
     eeprom_next_page=(eeprom_address+bytes_left-1)/128;                // see if  this is the last write
     if(eeprom_next_page==eeprom_curr_page)                             // if it is
        {bytes_to_read=bytes_left;
         bytes_left=0;
        }                                                               
     else {eeprom_end_page=eeprom_curr_page*128+127;                    // otherwise calcuate how many
           bytes_to_read=eeprom_end_page-eeprom_address+1;              // bytes to read
           bytes_left=bytes_left-bytes_to_read;                         // decrement bytes left
          }
      k_sleep(K_MSEC(4));                                               // sleep a few milliseconds

      ret=i2c_write_read(i2c1,ADDR,write_buffer,2,                // and go read 
                          read_buffer,bytes_to_read);                        
  
    //   ret=i2c_write(i2c1,buffer,bytes_to_read,EEPROMADDR);             // and write the buffer out to the EEPROM
      if(ret!=0)
           {
                 sprintf(buffer,"EEPROM READ FAIL AT %d",address);
                 bt_send_data(buffer);
                 if(!DUMMYREAD)
                    error_flash(0,1,0);
           }
      eeprom_address=eeprom_address+bytes_to_read;
      memmove(data,read_buffer,bytes_to_read);
      data=data+bytes_to_read;
    }
}



void eeprom_init()
{  uint32_t i;
   struct EEPROMMSGS MSG;
   eeprom_store(SIG_ADDRESS,SIG_SIZE,SIG_CONTENT);                       // go write the signature
   MAP.addr[0]=SIG_ADDRESS;                                              // initialize the EEPROM MAP - 0 is the SIGNATURE ADDRESS
   MAP.addr[1]=1024;                                                     // this is the cacert address in EEPROM
   MAP.addr[2]=0;                                                        // this is the actual length of the cacert - filled in when the certr is loaded
   MAP.addr[3]=4096;                                                     // this is the clicert address in EEPROM
   MAP.addr[4]=0;                                                        // this is the actual length of the clicert - filled in when the 
   MAP.addr[5]=7168;                                                     // this is the address of the clikey in EEPROM
   MAP.addr[6]=0;                                                        // this is the actual length of the clikey  - filled in whenthe key is loaded
   MAP.addr[7]=10240;                                                    // this is the AWS IOT endpoint address in EEPROM
   MAP.addr[8]=0;                                                        // this is the length of the AWS IOT endpiont - filled in when the endpoint is loaded
   MAP.addr[9]=128;                                                      // the PARAMS address
   MAP.addr[10]=46720;                                                   // this is the beginning of the LOGFILE  - there is room for 566 32 byte records  plus the section EEPROMMSGS map
   MAP.addr[11]=10368;                                                   // this is the beginning of the  ACTIVITY FILE - there us room for 566 32 byte records plus the section EEPROMMSGS map
   MAP.addr[12]=28544;                                                   // this is the beginning of the EERROR FILE - there is room for 566 32 byte records 1 plus the section EEPROMMSGS map
   eeprom_store(0,sizeof(MAP),(void *)&MAP);                             // store the EEPROM MAP at the beginning of the EEPROM
   for(i=10;i<13;i++)                                                    // now - built the LOG, ACTIVITY and ERROR file EEPROMMSGS blocks
   {
      MSG.fstmsg=9999;
      MSG.lstmsg=9999;
      MSG.msgcnt=0;
      if(i==10)
            MSG.availcnt=566;
      if(i==11)
          MSG.availcnt=566;
      if(i==12)
          MSG.availcnt=566;
          
      eeprom_store(MAP.addr[i],sizeof(MSG),(void *)&MSG);
      bt_send_data("CRAP");
   }
   memset(&PARMS,0,sizeof(PARMS));                                   // clear the params rcord
   PARMS.commdt=dt_to_posix(get_rtc());                              // setting the date to today
   strcpy(PARMS.lockid,"ZB00000001");                                // a dummy lock id
   strcpy(PARMS.crkey,"ABCDEFGH");                                   // and crypt key
   PARMS.ADVINT=2000;                                                // and the various params field
   PARMS.LOWVOLTAGE=4500;
   PARMS.ERRLMT=3;
   PARMS.ERRTMO=1;
   PARMS.INATMO=30;
   PARMS.RATIO=365;
   PARMS.LOCKSTATE='X';
   PARMS.USESOUND='Y';
   PARMS.USERFID='N';
   PARMS.USEAWS='Y';
   PARMS.TIMEZONE='5';
   PARMS.MOTORMODE='1';
   PARMS.TIMEOUTDELAY=10800;
   memset(PARMS.FWVERSION,0,sizeof(PARMS.FWVERSION));
   strcpy(PARMS.FWVERSION,FWVERSION);
   eeprom_store(MAP.addr[9],sizeof(PARMS),(void *)&PARMS);          // write the parram record
                     


}
