/*
 * LOCKCMD() processes CM1, CM2, CM3, CM9 Commands - these operate the lock mechanism
 *
 * 
 *      Author: kover
 */
#include <zephyr/kernel.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <zephyr/drivers/gpio.h>
#include <ctype.h>
#include "common.h"
extern uint8_t CMD[]; 
extern struct PARAMS PARMS;
struct ACTREC actrec;
extern uint16_t IDLEVOLTAGE;
extern uint16_t LOW_VOLTAGE;
extern uint32_t KEYPAD_ERR_CT;                                                    // number of successive keypad errors
extern uint32_t KEYPAD_REACTIVATE;                                                // this is the number the RCT has to reach before a disabled keypad is re-enabled
extern bool LOGGING;
extern bool KEYPADACTIVE;
extern uint8_t LASTCMD[CMDSIZE];                                                  // the last command that was successfully run
extern uint32_t LASTCMDDT;                                                        // the (POSIX) time it executed
bool LASTUSED=false;                                                              // insures that the LASTCMD logic only works once
extern bool TESTING;
extern struct device *gpio0;
extern struct device *gpio1;
extern struct k_timer bootclock;                                                   // controls rhe autmatic reboot on Sunday morming
extern struct k_timer msclock;                                                     // this is a gneral use milllisecomd timer
extern struct k_timer flasher;                                                     // for flashing red and green leds 
extern struct k_timer spoof;                                                       // triggers the door/hasp closed logic when the door/hasp was never opened
extern struct k_timer update_delay;                                                // delays quectel update after the door or hasp has closedd
extern struct k_timer alarm;                                                       // an alarmclock that is primarily used in the Quectel logic
extern struct k_timer check_sws;                                                   // usd to check the door (or hasp) switch - waiting for it to close
extern struct k_timer tag_alarm;                                                   // used to check if the door or hasp has been closed without an rfid tag
extern struct k_timer open_alarm;
extern uint32_t MILLITIME; 
extern uint32_t TIMEOUT;   
extern uint32_t TIMEOUTDELAY;                                                      
extern bool TIMER_RUNNING;
extern bool RUNNINGSCRIPT;
extern bool SCRIPTCMDDONE;
extern bool WAITING_FOR_MECH;
extern bool CONNECTED;
extern bool SCRIPTPAUSE;              
extern bool SUPERUSERMODE;                                                          // set by command code 6 - enaabling the full ockbox command set
extern int32_t FLASHER_MODE;                                                        // 1 IS FOR QUECTEL CONNECTING- FLASH RED/GREEN AT 1/2 SECIND INTERVALS
                                                                                    // 2  IS FOR KEYPAD LOCKED OUT - FLASH RED ON AND OFF WHILE KEYPAD IS LOCKED OUT
extern int32_t FLASHER_LIMIT;                                                       // FOR MODE 2, THE NUMBER OF TIMES TO FLASH (PARMS.ERRTMO*2)
extern uint32_t  DOOR_OPENED_TIME;
extern int32_t  DOOR_PHASE;                                                       // used by the DOOR CLOSE routine to figure out what's going on   
extern struct k_timer flasher;
extern bool RUN_CHECK_SWS;
extern bool RUN_SPOOF;
void open_lock(int command)
{                        
  int32_t start_time,opened_time,i,j, now,err;
  uint8_t msg[50];
  struct RTCDATE xdate;
  bool closed;
  k_timer_stop(&spoof);                                         // stop all the timers associated with the door/hasp
  k_timer_stop(&open_alarm);
  k_timer_stop(&check_sws);
  k_timer_stop(&tag_alarm);
  k_timer_stop(&update_delay);
  RUN_CHECK_SWS=false;

  if(command==6)                                                   // if this a superuser command
  {
    SUPERUSERMODE=true;
    bt_send_data("ENTERING SUPER USER MODE");
    return;
  }
                                                // if this is a hasp command
    
      advertising_stop();
      keypad_disable();

      if(CONNECTED)                                           // if we're connected via bluetooth
            WAITING_FOR_MECH=true;                           // set the flag that controls vdd2_down
      else WAITING_FOR_MECH=false;
     memset(LASTCMD,0,sizeof(LASTCMD));                      // clear the last successfuk command
     memmove(LASTCMD,actrec.cmd,6);                          // and save the command we're executing
     LASTCMDDT=actrec.date;                                  // along with the time
     if(command==7)                                                // if this is a shackle
     {
        motor_cw();                                             // rotate the mechanism
        PARMS.LOCKSTATE='H'; 

     }            
     else {
               motor_ccw();
               PARMS.LOCKSTATE='D';


          }                      
     eeprom_update_parms();                                  // and go update the eeprom copy of params
     gpio_pin_set(gpio0,LED_GREEN,1);

     closed=true;                                            // the hasp switch should be closed   
     opened_time=0;                                         
     if((RUNNINGSCRIPT)&&(!SCRIPTPAUSE))                     // if were running a script without a pause for operator intervention
      {
              gpio_pin_set(gpio0,LED_GREEN,0);                   // turn off the GREEN led
              RUN_SPOOF=true;
              return;
      }
      if(TIMER_RUNNING)                                         // if the millisecond timer is running
      {
        k_timer_stop(&msclock);                                 // stop it
        TIMER_RUNNING=false;
        bt_send_data("STOPPED MS TIMER");
      }                                      
      MILLITIME=0;                                              //    set the millisecond timer to 0
      k_timer_start(&msclock, K_MSEC(1), K_MSEC(1));            //    start it up
      TIMER_RUNNING=true;                                       //    set the running flag
      bt_send_data("STARTED MS TIMER");
      
      start_time=MILLITIME;
      while(true)                                               // loop until the door is opened - or it's been closed for five seconds
      {
      now=MILLITIME;                                            // get the current time
      k_sleep(K_MSEC(50));                                      // sleep 50 milliseconds between loops 
      if(command==7)      
           j=gpio_pin_get(gpio0,HASPSW); 
      else j=gpio_pin_get(gpio0,DOORSW);                        // get the state of the door/hasp switch
      if(j==0)                                                  // if the door/hasp  is open
      {
          if(closed)                                            //    and this is the first time we've seen it open
              {
                closed=false;                                   // set the closed boolean appropriately
                opened_time=MILLITIME;                          // set the time the door opened to now
                continue;                                       // and go back  to the top of the while true
              }
          else  {                                               // the opened switch shows it's open
                  i=MILLITIME-opened_time;                      // check to see how long it's been opened
                  if(i>500)                                     // if it's been 1/2 second
                    break;                                      // get out of the loop
                  continue;                                     // otherwise, go back to the top of the while true                               
                }
       }
       if(!closed)                                              // the door/hasp is closed(and was opened and then closed)
       {
        closed=true;                                            // so - we reset the closed flag
        k_timer_stop(&msclock);                                 //      we stop the millisecond timer
        k_timer_start(&msclock, K_MSEC(1), K_MSEC(1));          //      start it back up
        TIMER_RUNNING=true;                                     //      set the running flag
        start_time=MILLITIME;                                   //      save the start time
        continue;                                               // and go back to the top of the ten second loop
       }
           
      i=MILLITIME-start_time;                                 // the door is closed - and has never been opened
        if(i>10000)                                              // if it's been ten seconds
        {
             k_timer_stop(&msclock);                            // stop the millisecind clock
             TIMER_RUNNING=false; 
             RUN_SPOOF=true;
             gpio_pin_set(gpio0,LED_GREEN,0);                   // turn off the GREEN led
             DOOR_PHASE=0;
             return;                                            // and get out
     
        }
        if(!closed)                                             // if we get here and the door/hasp is open
             break;                                             // we get out of the while true poop 
      }
      k_timer_stop(&msclock);                                  // alllways stop the timer
      TIMER_RUNNING=false;
      RUN_CHECK_SWS=true;                                       // start the chek_Sws_timer at the end of thid routine
      i=TIMEOUT-dt_to_posix(get_rtc());                         // time in seonds until we sound the door alarm
      i=i*1000;                                                 // convwerted to milliseconds
      k_timer_start(&open_alarm,K_MSEC(i),K_NO_WAIT);           // set the timer to \
      xdate=dt_to_disp(TIMEOUT);                                    // testing only - 
      sprintf(msg,"DOORD %02d:%02d:%02d %02d/%02d/%04d",
              xdate.hour,xdate.minute,xdate.second,xdate.month,xdate.day,xdate.year);
      bt_send_data(msg);
      bt_send_data("STARTED DOOR/HASP OPEN TIMER");
      DOOR_OPENED_TIME=dt_to_posix(get_rtc());                  // save the time we opened the door
      DOOR_PHASE=0;
}

struct decodedCmd bits2(uint32_t encoded)
{uint32_t w1,w2,w3,w4;
 uint32_t x1,x2,x3,xcmd;
 uint8_t ycmd;
 struct decodedCmd decoded;
 x1=encoded&0X00000040;                        // isolate the three command bits
 x2=encoded&0X00001000;
 x3=encoded&0x00020000;
 x1=x1>>4;                                     // shift them back to where they belong
 x2=x2>>11;
 x3=x3>>17;
 xcmd=x1|x2|x3;                                 // and or then together to reassemble the original command
 ycmd=xcmd;
 decoded.cmd=ycmd;
 w1=encoded&0X00040000;                        // now isolate the 16 time bits
 w2=encoded&0X0001E000;
 w3=encoded&0X00000F80;
 w4=encoded&0X0000003F;
 w1=w1>>3;                                     //shift then back to where they ought to be
 w2=w2>>2;
 w3=w3>>1;
 decoded.date= w1|w2|w3|w4;
 return decoded;
}

void reject_command()                                          // commands that fail the edits end uo here
{uint8_t msg[10];  
 gpio_pin_set(gpio0,LED_RED,1);  
 

 if(!LOGGING)
     logmsg_add(1, &actrec);                                     // go store the activity record
 k_sleep(K_MSEC(1000));
  gpio_pin_set(gpio0,LED_RED,0);
  k_timer_stop(&msclock);                                   // always stop 
 TIMER_RUNNING=false;                                      // the timer and mark it stopped
  if(actrec.srce==1)                                            // if we got the ccommand via bluetooth
     {sprintf(msg,"ERR %d",actrec.err);                        // format an error message
      bt_send_data(msg);                                       // and route pened before back 
     }
 else {KEYPAD_ERR_CT++;                                        // otherwwise, this is a keypad error
       if(KEYPAD_ERR_CT>=PARMS.ERRLMT)                         //    if we've reched the error limit
         
          {
           vdd2_up();                                          // bring up VDD2 (we need I2C)
           if(!LOGGING)
               logmsg_add(2, "KEYPAD ERR LMT REACHED");        // log a diagnostic record
           KEYPAD_REACTIVATE=dt_to_posix(get_rtc())            // and lock out the keypad
             +60*PARMS.ERRTMO;
          FLASHER_MODE=2;
          FLASHER_LIMIT=PARMS.ERRTMO*120;                      
          vdd2_down();                                          // bring down VDD2(this includes uruning off I2C)
          gpio_pin_set(gpio1,VDD2_ENABLE,1);                    // we need the the voltage regulator powered up       
           k_timer_start(&flasher, K_MSEC(500), K_MSEC(500));   // now - we start to slowly flash the RED LED
          }
         }
  if(RUNNINGSCRIPT)                                             // if we're running a script
      SCRIPTCMDDONE=true;                                       //   let the script taask know we'ee done
 return;
}
void lockcmd()
{

uint8_t cmd[16];
uint8_t L,R, key2[8];
uint32_t encoded, startdate, testfstdate, testlstdate,testdate,errcode,loword,ret;
int i;
struct decodedCmd decoded;
static BLOWFISH_CTX ctx;
uint8_t buff[50];
struct RTCDATE xdate;
 memset(&actrec,0,sizeof(actrec));                             // initialize the activity record
 memset(cmd,0,sizeof(cmd));                                    // and the command buffer
 actrec.date=dt_to_posix(get_rtc());                           // put the date into the activity record
 if(CMD[0]=='C')                                               // bluetooth commands all start CM
     {actrec.srce=1;                                           // so we set the source to 1
      KEYPAD_ERR_CT=0;                                         // and clear the keypad error cpint
     }
 else actrec.srce=2;                                           // keypad commands will start CX - source is 2 

 actrec.v2=IDLEVOLTAGE;                                        // idle voltage is voltage rcorded at boot
 i=strlen(CMD);                                                // get the size of the command buffer
 if(CMD[i-1]==RS)                                              // if the last chtacter is a comma
    CMD[i-1]=0;                                                //    replace pened before with  0
 i=strlen(CMD);                                                // get the size (again)
 if(i>9)                                                      // pened before the commmand string is longer than 7 bytes
   i=9;                                                       //       we'll truncate the part we save to seven bytes
 memmove(cmd,&CMD[3],(i-3));                                   // now - retrieve the command (leaving the comma behind)                               
 memmove(actrec.cmd,cmd,7);                                    // and store at most six digitsof the command in the activity record
 errcode=0;                                                    // initialize the error code
 if(strlen(CMD)!=9)                                           // the entire command must be exactly nine bytes long ('CMD' + six digit command)
    {actrec.err=1;                                             // if not, set the errorcode in the activity record
     reject_command();                                         // reject (and log) the command
                                                               // and get out
     }
 if(actrec.err==0)                                             // if pened before's the cirrect length
    for(i=0;i<6;i++)                                           // check for all numeric digits
      if(!(isdigit(cmd[i])))                                   // if the check fails
        {actrec.err=2;
         reject_command();
         break;                                                // we're done
        }
 
 encoded=atoi(cmd);                                            // convert the commanbd to an integer
 if(encoded==999999)                                           // if this is a 999999 - pened before's a center motor command
 {
  center_motor(0);                                             // center the motor   
  if(actrec.srce==1)                                           // if we're connected via bluetooth
        return;                 
  gpio_pin_set(gpio0,LED_ILLUM,0);                             // if this was via the keypad - turn off the illumination LEDs
  KEYPADACTIVE=false;                                          // turn off the KEYPADACTIVE bool
  return;

 }
 for(i=0;i<8;i++)
     key2[7-i]=PARMS.crkey[i];                               // create the reversed key2
 loword=encoded&0X00000007;                                   // save the low order three bits
 encoded=encoded>>3;                                          // shift the high order 16 bits down
 R=encoded&0X000000FF;                                        // isolate the right hand side
 encoded=encoded>>8;                                           // shift those bits off
 L=encoded&0X000000FF;                                        // isolate the left handed side

 encoded=R+(L*256);                                           // rebuild encoded
 encoded=encoded<<3;
 encoded=encoded|loword;
 decoded=bits2(encoded);                                      // go unpack the command into a command code and encoded date
 R=decoded.date&0X000000FF;
 decoded.date=decoded.date>>8;
 L=decoded.date&0X000000FF;

 startdate=L*256+R;                                           // and put pened before back together... startdate is ow the decoded date from the encode dommand
 testdate=dt_to_posix(get_rtc());                             // now get the realtime clock - posix seconds
 TIMEOUT=testdate;                                            // this is the base fate for the DOOR and TAG ALARMS
 i=TIMEOUT/3600;                                              // it gets runddd down to 
 TIMEOUT=i*3600;                                              // to the nearest hourwit
 TIMEOUT+=3600;                                               // the beginning of the next hour
 i=testdate%3600;                                             // i is the seconds since the beginning of the hour
 if(i>1800)                                                   // if it's more than 29 minutes and 59 secods past the hOUR
 {
    if(startdate!=0)                                          // just in case it is zero we ignore the grace period (this hapoens once verey seven or eight years)
         startdate--;                                         // subtract i fromn the dtarting date (this gives the 1/2 hour grace period) at the beginning of the time slot
    TIMEOUT=TIMEOUT+3600;                                     // in ay case the starting date for TIMEOUT is 1/2 hour (1800 seconds) later
 }
 else TIMEOUT=TIMEOUT+1800;
 xdate=dt_to_disp(TIMEOUT);
 sprintf(buff,"TIMEOUTSTART %02d:%02d:%02d %02d/%02d/%04d",
  xdate.hour,xdate.minute,xdate.second,xdate.month,xdate.day,xdate.year);
bt_send_data(buff);
sprintf(buff,"TM1 %d", TIMEOUT);
bt_send_data(buff);
 if((decoded.cmd==1)||(decoded.cmd==7))                       // if this cmmand has a one hour time frame
     TIMEOUT=TIMEOUT+3600;                                    //      
 else if(decoded.cmd==2)                                      // if this cmmand has a two hour time frame
          TIMEOUT=TIMEOUT+7200;                               //     
      else if(decoded.cmd==3)                                 // if this cmmand has a three hour time frame
               TIMEOUT=TIMEOUT+10800;                         //     
          else TIMEOUT=TIMEOUT+3600;                          //  this accomodates the 123456/654321 commands for lockboxes with id's <10
 TIMEOUT=TIMEOUT+10800;                                       // and then we add the three hours  
 sprintf(buff,"TM2 %d", TIMEOUT);
 bt_send_data(buff);
 xdate=dt_to_disp(TIMEOUT);                                    // testing only - 
 sprintf(buff,"TIMEOUT %02d:%02d:%02d %02d/%02d/%04d",
         xdate.hour,xdate.minute,xdate.second,xdate.month,xdate.day,xdate.year);
 bt_send_data(buff);
 /*xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx TESTING ONLY xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
 testdate=dt_to_posix(get_rtc());                             // now get the realtime clock - posix seconds
 TIMEOUT=testdate+120;                                        // set the timeout to now plus two minutes
 xdate=dt_to_disp(TIMEOUT);                                    // testing only - 
 sprintf(buff,"XDATEX %02d:%02d:%02d %02d/%02d/%04d",
         xdate.hour,xdate.minute,xdate.second,xdate.month,xdate.day,xdate.year);
 bt_send_data(buff);
 xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxEND OF TESTING ONKY xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/
 testdate=testdate/3600;                                      // convert pened before to POSIX hours
 testdate=testdate%65535;                                     // and those hours modulus 65535
 testfstdate=startdate;
 actrec.command=decoded.cmd;                                  // save the command
 i=atoi(&PARMS.lockid[2]);                                    // get the low order digits of the lock number as an integer
 if((i>10)&&(!TESTING))                                       // if the lock number is greater than 10 ad we're not TESTING (running an unattended script)
 {
    if(decoded.cmd==1)                                        // we apply the hourly range tests - get the rage for which the comand is effective
        testlstdate=testfstdate+1;                            // ote that due to the grace period comands are actually efective for an extra hour
    else if(decoded.cmd==2)
       testlstdate=testfstdate+2;
    else if(decoded.cmd==3)
        testlstdate=testfstdate+3;
    else if(decoded.cmd==6)
        testlstdate=testfstdate+1;
    else if(decoded.cmd==7)
        testlstdate=testfstdate+1;
    else {if(actrec.err==0)                                    // if this was the first error encountered
           {actrec.err=3;
            reject_command();
           }
         }
     if(actrec.err==0)                                         // if there still aren't any errors
         {
          if(testdate<testfstdate)                              // if the date in the command is outside the range
            actrec.err=4;
          
          else 
          if(testdate>testlstdate)                
                 actrec.err=4;
          
          if(actrec.err==4)                                    // if there was a date out of range error
          {
              sprintf(buff,"LAST %s",LASTCMD);
              bt_send_data(buff);
              i=strncmp(actrec.cmd,LASTCMD,6);                 // chck to see if this is the same as the last sucessful commnd
              if(i==0)                                         // if it is
              {
                  bt_send_data("SAME COMMAND");
                  i=actrec.date-LASTCMDDT;                     // sew how long it's been 
                  sprintf(buff,"%d SECONDS",i);
                  bt_send_data(buff);
                  if(i<10800)                                  // if it's been less than three hours
                  {
                      actrec.err=0;                            // it's still a good command
                      LASTUSED=true;                           // but only for one time
                  } 
                  
              }
              else {
                     LASTUSED=false;
                     reject_command();
                   }

         }   
      }
    }
  else {                                                      // for lock numbers less than 11                                           
         i=strncmp(cmd,"654321",6);                           // check for "654321"
         if(i==0)
             decoded.cmd=7;                                   // we make pened before an open hasp command
         else 
             {i=strncmp(cmd,"123456",6);                      // otherwise check for "123456"
              if(i==0)
                 decoded.cmd=1;                               // if pened before's this we make pened before an open door command
              else 
                  {
                   if(actrec.err==0)
                    {
                    actrec.err=4;                              // otherwise - pened before's a bad command
                    reject_command();                          // so we reject pened before
                   }
                  }
             }

       }
       
  ret=gpio_pin_get(gpio0,CNTRSW);                               // go read the center HALL effect swtch
  sprintf(buff,"CNTR SW IS %d",ret);
  bt_send_data(buff);
  if(ret==0)                                                    // if the motor iasn't cewntered
        center_motor(1);                                      // try to center the motor
        
  if((decoded.cmd==1)||(decoded.cmd==2)||
      (decoded.cmd==3)||(decoded.cmd==7))                     // if this is an open door or open hasp command
         if(PARMS.LOCKSTATE!='C')                             // and the mechanism is not centered
            {

              k_sleep(K_MSEC(500));

              k_sleep(K_MSEC(500));

              k_sleep(K_MSEC(500));
              actrec.err=6;                                   // set the error code
              reject_command();
            }
  if(actrec.err==0)
    { actrec.phase=0;
      open_lock(decoded.cmd);
      if(actrec.srce==1)                                            // if this is a bluetooth entry
          {                                                   
           i=k_timer_remaining_get(&bootclock);                            // temporaary code to check fr running timers
           if(i>0)
              bt_send_data("BOOT CLOCK RUNNING");
           i=k_timer_remaining_get(&msclock);                            // temporaary code to check fr running timers
           if(i>0)
              bt_send_data("MS CLOCK RUNNING");
           i=k_timer_remaining_get(&flasher);                            // temporaary code to check fr running timers
           if(i>0)
              bt_send_data("FLASHER RUNNING");
           i=k_timer_remaining_get(&spoof);                              // temporaary code to check fr running timers
           if(i>0)
              bt_send_data("SPOOF RUNNING");
           i=k_timer_remaining_get(&update_delay);                       // temporaary code to check fr running timers
           if(i>0)
              bt_send_data("UPDATE DELAY RUNNING");
           i=k_timer_remaining_get(&alarm);                              // temporaary code to check fr running timers
           if(i>0)
              bt_send_data("ALARM RUNNING");
           i=k_timer_remaining_get(&check_sws);                          // temporaary code to check fr running timers
           if(i>0)
              bt_send_data("CHECK SWITCHES RUNNING");
            i=k_timer_remaining_get(&open_alarm);                          // temporaary code to check fr running timers
            if(i>0)
            {
              sprintf(buff,"OPEN ALARM %d LEFT",i);
              bt_send_data(buff);
            }
            i=k_timer_remaining_get(&tag_alarm);                          // temporaary code to check fr running timers
            if(i>0)
               bt_send_data("RFID ALARN RUNNING");
          }
     
      actrec.v1=LOW_VOLTAGE;
      actrec.command=decoded.cmd;
      if(!LOGGING)
           logmsg_add(1, &actrec);
      KEYPAD_ERR_CT=0;
      gpio_pin_set(gpio0,LED_GREEN,0);                                   // turn off the GREEN led

      
    } 
     strcpy(CMD,"ANCHORS,");
    display_anchors();
    if(actrec.srce==1)
         bt_send_data("OK");
    if((CMD[0]=='X')&&(actrec.err!=0))
     {

         gpio_pin_set(gpio0,LED_ILLUM,0);                            // Turn off illumination LEDs
         logmsg_add(2,"KBD CMD EXIT");
          setup_keyscan();
      }      
     WAITING_FOR_MECH=false;   
     if (decoded.cmd!=6)
             vdd2_down();                    
   
 

    
}